@@ -27,14 +27,15 @@
 --
 -- > {-# LANGUAGE OverloadedStrings #-}
 -- > import Control.Monad.Trans.Resource
--- > import Data.Conduit (($$))
+-- > import Data.Conduit (Consumer, Sink, ($$))
 -- > import Data.Text (Text, unpack)
 -- > import Text.XML.Stream.Parse
+-- > import Data.XML.Types (Event)
 -- >
 -- > data Person = Person Int Text
 -- >     deriving Show
 -- >
--- > parsePerson :: MonadThrow m => Consumer Event m (Maybe Person)
+-- > parsePerson :: MonadThrow m => ConduitT Event o m (Maybe Person)
 -- > parsePerson = tag' "person" (requireAttr "age") $ \age -> do
 -- >     name <- content
 -- >     return $ Person (read $ unpack age) name
@@ -71,7 +72,7 @@
 -- >
 -- > data Person = Person Int Text deriving Show
 -- >
--- > parsePerson :: MonadThrow m => Consumer Event m (Maybe Person)
+-- > parsePerson :: MonadThrow m => ConduitT Event o m (Maybe Person)
 -- > parsePerson = tag' "person" (requireAttr "age") $ \age -> do
 -- >     name <- content
 -- >     return $ Person (read $ unpack age) name
@@ -80,7 +81,7 @@
 -- > parsePeople = void $ tagNoAttr "people" $ manyYield parsePerson
 -- >
 -- > main = runResourceT $
--- >     parseFile def "people.xml" $$ parsePeople =$ CL.mapM_ (lift . print)
+-- >     parseFile def "people.xml" $$ parsePeople .| CL.mapM_ (lift . print)
 --
 -- Previous versions of this module contained a number of more sophisticated
 -- functions written by Aristid Breitkreuz and Dmitry Olshansky. To keep this
@@ -100,7 +101,9 @@ module Text.XML.Stream.Parse
     , ParseSettings
     , def
     , DecodeEntities
+    , DecodeIllegalCharacters
     , psDecodeEntities
+    , psDecodeIllegalCharacters
     , psRetainNamespaces
       -- *** Entity decoding
     , decodeXmlEntities
@@ -156,19 +159,18 @@ module Text.XML.Stream.Parse
     , PositionRange
     , EventPos
     ) where
-import           Blaze.ByteString.Builder     (fromWord32be, toByteString)
-import           Control.Applicative          ((<$>))
 import           Control.Applicative          (Alternative (empty, (<|>)),
                                                Applicative (..), (<$>))
 import qualified Control.Applicative          as A
 import           Control.Arrow                ((***))
 import           Control.Exception            (Exception (..), SomeException)
-import           Control.Monad                (ap, guard, liftM, void)
+import           Control.Monad                (ap, liftM, void)
 import           Control.Monad.Fix            (fix)
+import           Control.Monad.IO.Class       (liftIO)
 import           Control.Monad.Trans.Class    (lift)
 import           Control.Monad.Trans.Maybe    (MaybeT (..))
 import           Control.Monad.Trans.Resource (MonadResource, MonadThrow (..),
-                                               monadThrow)
+                                               throwM)
 import           Data.Attoparsec.Text         (Parser, anyChar, char, manyTill,
                                                skipWhile, string, takeWhile,
                                                takeWhile1, try)
@@ -176,12 +178,10 @@ import qualified Data.Attoparsec.Text         as AT
 import qualified Data.ByteString              as S
 import qualified Data.ByteString.Lazy         as L
 import           Data.Char                    (isSpace)
-import           Data.Conduit
-import           Data.Conduit.Attoparsec      (PositionRange, conduitParser)
-import           Data.Conduit.Binary          (sourceFile)
-import qualified Data.Conduit.List            as CL
+import           Conduit
 import qualified Data.Conduit.Text            as CT
-import           Data.Default                 (Default (..))
+import           Data.Conduit.Attoparsec      (PositionRange, conduitParser)
+import           Data.Default.Class           (Default (..))
 import           Data.List                    (intercalate)
 import           Data.List                    (foldl')
 import qualified Data.Map                     as Map
@@ -189,12 +189,9 @@ import           Data.Maybe                   (fromMaybe, isNothing)
 import           Data.String                  (IsString (..))
 import           Data.Text                    (Text, pack)
 import qualified Data.Text                    as T
-import           Data.Text.Encoding           (decodeUtf32BEWith,
-                                               decodeUtf8With)
-import           Data.Text.Encoding.Error     (ignore, lenientDecode)
-import           Data.Text.Read               (Reader, decimal, hexadecimal)
+import           Data.Text.Encoding           (decodeUtf8With)
+import           Data.Text.Encoding.Error     (lenientDecode)
 import           Data.Typeable                (Typeable)
-import           Data.Word                    (Word32)
 import           Data.XML.Types               (Content (..), Event (..),
                                                ExternalID (..),
                                                Instruction (..), Name (..))
@@ -273,7 +270,7 @@ tnameToName _ (NSLevel _ m) (TName (Just pref) name) =
 -- first checks for BOMs, removing them as necessary, and then check for the
 -- equivalent of <?xml for each of UTF-8, UTF-16LE/BE, and UTF-32LE/BE. It
 -- defaults to assuming UTF-8.
-detectUtf :: MonadThrow m => Conduit S.ByteString m T.Text
+detectUtf :: MonadThrow m => ConduitT S.ByteString T.Text m ()
 detectUtf =
     conduit id
   where
@@ -309,10 +306,10 @@ detectUtf =
 checkXMLDecl :: MonadThrow m
              => S.ByteString
              -> Maybe CT.Codec
-             -> Conduit S.ByteString m T.Text
+             -> ConduitT S.ByteString T.Text m ()
 checkXMLDecl bs (Just codec) = leftover bs >> CT.decode codec
 checkXMLDecl bs0 Nothing =
-    loop [] (AT.parse (parseToken decodeXmlEntities)) bs0
+    loop [] (AT.parse (parseToken def)) bs0
   where
     loop chunks0 parser nextChunk =
         case parser $ decodeUtf8With lenientDecode nextChunk of
@@ -345,15 +342,15 @@ type EventPos = (Maybe PositionRange, Event)
 -- to do the actual parsing.
 parseBytes :: MonadThrow m
            => ParseSettings
-           -> Conduit S.ByteString m Event
+           -> ConduitT S.ByteString Event m ()
 parseBytes = mapOutput snd . parseBytesPos
 
 parseBytesPos :: MonadThrow m
               => ParseSettings
-              -> Conduit S.ByteString m EventPos
-parseBytesPos ps = detectUtf =$= parseTextPos ps
+              -> ConduitT S.ByteString EventPos m ()
+parseBytesPos ps = detectUtf .| parseTextPos ps
 
-dropBOM :: Monad m => Conduit T.Text m T.Text
+dropBOM :: Monad m => ConduitT T.Text T.Text m ()
 dropBOM =
     await >>= maybe (return ()) push
   where
@@ -376,13 +373,13 @@ dropBOM =
 -- Since 1.2.4
 parseText' :: MonadThrow m
            => ParseSettings
-           -> Conduit T.Text m Event
+           -> ConduitT T.Text Event m ()
 parseText' = mapOutput snd . parseTextPos
 
 {-# DEPRECATED parseText "Please use 'parseText'' or 'parseTextPos'." #-}
 parseText :: MonadThrow m
           => ParseSettings
-          -> Conduit T.Text m EventPos
+          -> ConduitT T.Text EventPos m ()
 parseText = parseTextPos
 
 -- | Same as 'parseText'', but includes the position of each event.
@@ -390,12 +387,12 @@ parseText = parseTextPos
 -- Since 1.2.4
 parseTextPos :: MonadThrow m
           => ParseSettings
-          -> Conduit T.Text m EventPos
+          -> ConduitT T.Text EventPos m ()
 parseTextPos de =
     dropBOM
-        =$= tokenize
-        =$= toEventC de
-        =$= addBeginEnd
+        .| tokenize
+        .| toEventC de
+        .| addBeginEnd
   where
     tokenize = conduitToken de
     addBeginEnd = yield (Nothing, EventBeginDocument) >> addEnd
@@ -403,7 +400,7 @@ parseTextPos de =
         (yield (Nothing, EventEndDocument))
         (\e -> yield e >> addEnd)
 
-toEventC :: Monad m => ParseSettings -> Conduit (PositionRange, Token) m EventPos
+toEventC :: Monad m => ParseSettings -> ConduitT (PositionRange, Token) EventPos m ()
 toEventC ps =
     go [] []
   where
@@ -415,9 +412,13 @@ toEventC ps =
           where
             (es', levels', events) = tokenToEvent ps es levels token
 
+
+type DecodeEntities = Text -> Content
+type DecodeIllegalCharacters = Int -> Maybe Char
+
 data ParseSettings = ParseSettings
-    { psDecodeEntities   :: DecodeEntities
-    , psRetainNamespaces :: Bool
+    { psDecodeEntities          :: DecodeEntities
+    , psRetainNamespaces        :: Bool
     -- ^ Whether the original xmlns attributes should be retained in the parsed
     -- values. For more information on motivation, see:
     --
@@ -426,19 +427,29 @@ data ParseSettings = ParseSettings
     -- Default: False
     --
     -- Since 1.2.1
+    , psDecodeIllegalCharacters :: DecodeIllegalCharacters
+    -- ^ How to decode illegal character references (@&#[0-9]+;@ or @&#x[0-9a-fA-F]+;@).
+    --
+    -- Character references within the legal ranges defined by <https://www.w3.org/TR/REC-xml/#NT-Char the standard> are automatically parsed.
+    -- Others are passed to this function.
+    --
+    -- Default: @const Nothing@
+    --
+    -- Since 1.7.1
     }
 
 instance Default ParseSettings where
     def = ParseSettings
         { psDecodeEntities = decodeXmlEntities
         , psRetainNamespaces = False
+        , psDecodeIllegalCharacters = const Nothing
         }
 
-conduitToken :: MonadThrow m => ParseSettings -> Conduit T.Text m (PositionRange, Token)
-conduitToken = conduitParser . parseToken . psDecodeEntities
+conduitToken :: MonadThrow m => ParseSettings -> ConduitT T.Text (PositionRange, Token) m ()
+conduitToken = conduitParser . parseToken
 
-parseToken :: DecodeEntities -> Parser Token
-parseToken de = (char '<' >> parseLt) <|> TokenContent <$> parseContent de False False
+parseToken :: ParseSettings -> Parser Token
+parseToken settings = (char '<' >> parseLt) <|> TokenContent <$> parseContent settings False False
   where
     parseLt =
         (char '?' >> parseInstr) <|>
@@ -449,7 +460,7 @@ parseToken de = (char '<' >> parseLt) <|> TokenContent <$> parseContent de False
         name <- parseIdent
         if name == "xml"
             then do
-                as <- A.many $ parseAttribute de
+                as <- A.many $ parseAttribute settings
                 skipSpace
                 char' '?'
                 char' '>'
@@ -528,14 +539,14 @@ parseToken de = (char '<' >> parseLt) <|> TokenContent <$> parseContent de False
     parseBegin = do
         skipSpace
         n <- parseName
-        as <- A.many $ parseAttribute de
+        as <- A.many $ parseAttribute settings
         skipSpace
         isClose <- (char '/' >> skipSpace >> return True) <|> return False
         char' '>'
         return $ TokenBeginElement n as isClose 0
 
-parseAttribute :: DecodeEntities -> Parser TAttribute
-parseAttribute de = do
+parseAttribute :: ParseSettings -> Parser TAttribute
+parseAttribute settings = do
     skipSpace
     key <- parseName
     skipSpace
@@ -544,8 +555,8 @@ parseAttribute de = do
     val <- squoted <|> dquoted
     return (key, val)
   where
-    squoted = char '\'' *> manyTill (parseContent de False True) (char '\'')
-    dquoted = char  '"' *> manyTill (parseContent de True False) (char  '"')
+    squoted = char '\'' *> manyTill (parseContent settings False True) (char '\'')
+    dquoted = char  '"' *> manyTill (parseContent settings True False) (char  '"')
 
 parseName :: Parser TName
 parseName =
@@ -567,18 +578,42 @@ parseIdent =
     valid '"'  = False
     valid '\'' = False
     valid '/'  = False
+    valid ';'  = False
+    valid '#'  = False
     valid c    = not $ isXMLSpace c
 
-parseContent :: DecodeEntities
+parseContent :: ParseSettings
              -> Bool -- break on double quote
              -> Bool -- break on single quote
              -> Parser Content
-parseContent de breakDouble breakSingle = parseEntity <|> parseTextContent where
-  parseEntity = do
+parseContent (ParseSettings decodeEntities _ decodeIllegalCharacters) breakDouble breakSingle = parseReference <|> parseTextContent where
+  parseReference = do
     char' '&'
-    t <- takeWhile1 (/= ';')
+    t <- parseEntityRef <|> parseHexCharRef <|> parseDecCharRef
     char' ';'
-    return $ de t
+    return t
+  parseEntityRef = do
+    TName ma b <- parseName
+    let name = maybe "" (`T.append` ":") ma `T.append` b
+    return $ case name of
+      "lt"   -> ContentText "<"
+      "gt"   -> ContentText ">"
+      "amp"  -> ContentText "&"
+      "quot" -> ContentText "\""
+      "apos" -> ContentText "'"
+      _      -> decodeEntities name
+  parseHexCharRef = do
+    void $ string "#x"
+    n <- AT.hexadecimal
+    case toValidXmlChar n <|> decodeIllegalCharacters n of
+      Nothing -> fail "Invalid character from hexadecimal character reference."
+      Just c -> return $ ContentText $ T.singleton c
+  parseDecCharRef = do
+    void $ string "#"
+    n <- AT.decimal
+    case toValidXmlChar n <|> decodeIllegalCharacters n of
+      Nothing -> fail "Invalid character from decimal character reference."
+      Just c  -> return $ ContentText $ T.singleton c
   parseTextContent = ContentText <$> takeWhile1 valid
   valid '"'  = not breakDouble
   valid '\'' = not breakSingle
@@ -586,6 +621,24 @@ parseContent de breakDouble breakSingle = parseEntity <|> parseTextContent where
   valid '<'  = False -- lt
   valid _    = True
 
+-- | Is this codepoint a valid XML character? See
+-- <https://www.w3.org/TR/xml/#charsets>. This is proudly XML 1.0 only.
+toValidXmlChar :: Int -> Maybe Char
+toValidXmlChar n
+  | any checkRange ranges = Just (toEnum n)
+  | otherwise = Nothing
+  where
+    --Inclusive lower bound, inclusive upper bound.
+    ranges :: [(Int, Int)]
+    ranges =
+      [ (0x9, 0xA)
+      , (0xD, 0xD)
+      , (0x20, 0xD7FF)
+      , (0xE000, 0xFFFD)
+      , (0x10000, 0x10FFFF)
+      ]
+    checkRange (lb, ub) = lb <= n && n <= ub
+
 skipSpace :: Parser ()
 skipSpace = skipWhile isXMLSpace
 
@@ -613,13 +666,13 @@ data ContentType = Ignore | IsContent Text | IsError String | NotContent
 -- | Grabs the next piece of content if available. This function skips over any
 -- comments and instructions and concatenates all content until the next start
 -- or end tag.
-contentMaybe :: MonadThrow m => Consumer Event m (Maybe Text)
+contentMaybe :: MonadThrow m => ConduitT Event o m (Maybe Text)
 contentMaybe = do
-    x <- CL.peek
+    x <- peekC
     case pc' x of
-        Ignore      -> CL.drop 1 >> contentMaybe
-        IsContent t -> CL.drop 1 >> fmap Just (takeContents (t:))
-        IsError e   -> lift $ monadThrow $ InvalidEntity e x
+        Ignore      -> dropC 1 >> contentMaybe
+        IsContent t -> dropC 1 >> fmap Just (takeContents (t:))
+        IsError e   -> lift $ throwM $ InvalidEntity e x
         NotContent  -> return Nothing
   where
     pc' Nothing  = NotContent
@@ -636,16 +689,16 @@ contentMaybe = do
     pc EventInstruction{} = Ignore
     pc EventComment{} = Ignore
     takeContents front = do
-        x <- CL.peek
+        x <- peekC
         case pc' x of
-            Ignore      -> CL.drop 1 >> takeContents front
-            IsContent t -> CL.drop 1 >> takeContents (front . (:) t)
-            IsError e   -> lift $ monadThrow $ InvalidEntity e x
+            Ignore      -> dropC 1 >> takeContents front
+            IsContent t -> dropC 1 >> takeContents (front . (:) t)
+            IsError e   -> lift $ throwM $ InvalidEntity e x
             NotContent  -> return $ T.concat $ front []
 
 -- | Grabs the next piece of content. If none if available, returns 'T.empty'.
 -- This is simply a wrapper around 'contentMaybe'.
-content :: MonadThrow m => Consumer Event m Text
+content :: MonadThrow m => ConduitT Event o m Text
 content = fromMaybe T.empty <$> contentMaybe
 
 
@@ -679,9 +732,9 @@ tag :: MonadThrow m
     -> (a -> AttrParser b) -- ^ Given the value returned by the name checker, this function will
                            --   be used to get an @AttrParser@ appropriate for the specific tag.
                            --   If the @AttrParser@ fails, the function will also return @Nothing@
-    -> (b -> ConduitM Event o m c) -- ^ Handler function to handle the attributes and children
+    -> (b -> ConduitT Event o m c) -- ^ Handler function to handle the attributes and children
                                    --   of a tag, given the value return from the @AttrParser@
-    -> ConduitM Event o m (Maybe c)
+    -> ConduitT Event o m (Maybe c)
 tag nameMatcher attrParser f = do
   (x, leftovers) <- dropWS []
   res <- case x of
@@ -694,7 +747,7 @@ tag nameMatcher attrParser f = do
           case a of
             Just (EventEndElement name')
               | name == name' -> return (Just z')
-            _ -> lift $ monadThrow $ InvalidEndElement name a
+            _ -> lift $ throwM $ InvalidEndElement name a
       Nothing -> return Nothing
     _ -> return Nothing
 
@@ -722,29 +775,29 @@ tag nameMatcher attrParser f = do
         case runAttrParser p as of
             Left e          -> Left e
             Right ([], x)   -> Right x
-            Right (attr, _) -> Left $ toException $ UnparsedAttributes attr
+            Right (attr', _) -> Left $ toException $ UnparsedAttributes attr'
 
 -- | A simplified version of 'tag' where the 'NameMatcher' result isn't forwarded to the attributes parser.
 --
 -- Since 1.5.0
 tag' :: MonadThrow m
-     => NameMatcher a -> AttrParser b -> (b -> ConduitM Event o m c)
-     -> ConduitM Event o m (Maybe c)
+     => NameMatcher a -> AttrParser b -> (b -> ConduitT Event o m c)
+     -> ConduitT Event o m (Maybe c)
 tag' a b = tag a (const b)
 
 -- | A further simplified tag parser, which requires that no attributes exist.
 tagNoAttr :: MonadThrow m
           => NameMatcher a -- ^ Check if this is a correct tag name
-          -> ConduitM Event o m b -- ^ Handler function to handle the children of the matched tag
-          -> ConduitM Event o m (Maybe b)
+          -> ConduitT Event o m b -- ^ Handler function to handle the children of the matched tag
+          -> ConduitT Event o m (Maybe b)
 tagNoAttr name f = tag' name (return ()) $ const f
 
 
 -- | A further simplified tag parser, which ignores all attributes, if any exist
 tagIgnoreAttrs :: MonadThrow m
                => NameMatcher a -- ^ Check if this is a correct tag name
-               -> ConduitM Event o m b -- ^ Handler function to handle the children of the matched tag
-               -> ConduitM Event o m (Maybe b)
+               -> ConduitT Event o m b -- ^ Handler function to handle the children of the matched tag
+               -> ConduitT Event o m (Maybe b)
 tagIgnoreAttrs name f = tag' name ignoreAttrs $ const f
 
 
@@ -756,14 +809,14 @@ tagIgnoreAttrs name f = tag' name ignoreAttrs $ const f
 -- Since 1.5.0
 ignoreEmptyTag :: MonadThrow m
           => NameMatcher a -- ^ Check if this is a correct tag name
-          -> ConduitM Event o m (Maybe ())
+          -> ConduitT Event o m (Maybe ())
 ignoreEmptyTag nameMatcher = tagIgnoreAttrs nameMatcher (return ())
 
 
 {-# DEPRECATED ignoreTag "Please use 'ignoreEmptyTag'." #-}
 ignoreTag :: MonadThrow m
           => NameMatcher a -- ^ Check if this is a correct tag name
-          -> ConduitM Event o m (Maybe ())
+          -> ConduitT Event o m (Maybe ())
 ignoreTag = ignoreEmptyTag
 
 
@@ -774,21 +827,21 @@ ignoreTag = ignoreEmptyTag
 -- Since 1.5.0
 ignoreTreeContent :: MonadThrow m
                   => NameMatcher a -- ^ Check if this is a correct tag name
-                  -> ConduitM Event o m (Maybe ())
+                  -> ConduitT Event o m (Maybe ())
 ignoreTreeContent namePred = tagIgnoreAttrs namePred (void $ many ignoreAnyTreeContent)
 
 {-# DEPRECATED ignoreTree "Please use 'ignoreTreeContent'." #-}
 ignoreTree :: MonadThrow m
            => NameMatcher a -- ^ Check if this is a correct tag name
-           -> ConduitM Event o m (Maybe ())
+           -> ConduitT Event o m (Maybe ())
 ignoreTree = ignoreTreeContent
 
 -- | Like 'ignoreTreeContent', but matches any name and also ignores content events.
-ignoreAnyTreeContent :: MonadThrow m => ConduitM Event o m (Maybe ())
+ignoreAnyTreeContent :: MonadThrow m => ConduitT Event o m (Maybe ())
 ignoreAnyTreeContent = (void <$> contentMaybe) `orE` ignoreTreeContent anyName
 
 {-# DEPRECATED ignoreAllTreesContent "Please use 'ignoreAnyTreeContent'." #-}
-ignoreAllTreesContent :: MonadThrow m => ConduitM Event o m (Maybe ())
+ignoreAllTreesContent :: MonadThrow m => ConduitT Event o m (Maybe ())
 ignoreAllTreesContent = ignoreAnyTreeContent
 
 -- | Get the value of the first parser which returns 'Just'. If no parsers
@@ -796,16 +849,16 @@ ignoreAllTreesContent = ignoreAnyTreeContent
 --
 -- > orE a b = choose [a, b]
 orE :: Monad m
-    => Consumer Event m (Maybe a) -- ^ The first (preferred) parser
-    -> Consumer Event m (Maybe a) -- ^ The second parser, only executed if the first parser fails
-    -> Consumer Event m (Maybe a)
+    => ConduitT Event o m (Maybe a) -- ^ The first (preferred) parser
+    -> ConduitT Event o m (Maybe a) -- ^ The second parser, only executed if the first parser fails
+    -> ConduitT Event o m (Maybe a)
 orE a b = a >>= \x -> maybe b (const $ return x) x
 
 -- | Get the value of the first parser which returns 'Just'. If no parsers
 -- succeed (i.e., return 'Just'), this function returns 'Nothing'.
 choose :: Monad m
-       => [ConduitM Event o m (Maybe a)] -- ^ List of parsers that will be tried in order.
-       -> ConduitM Event o m (Maybe a)   -- ^ Result of the first parser to succeed, or @Nothing@
+       => [ConduitT Event o m (Maybe a)] -- ^ List of parsers that will be tried in order.
+       -> ConduitT Event o m (Maybe a)   -- ^ Result of the first parser to succeed, or @Nothing@
                                          --   if no parser succeeded
 choose []     = return Nothing
 choose (i:is) = i >>= maybe (choose is) (return . Just)
@@ -825,15 +878,15 @@ force msg i = i >>= maybe (throwM $ XmlException msg Nothing) return
 parseFile :: MonadResource m
           => ParseSettings
           -> FilePath
-          -> Producer m Event
-parseFile ps fp = sourceFile fp =$= parseBytes ps
+          -> ConduitT i Event m ()
+parseFile ps fp = sourceFile fp .| transPipe liftIO (parseBytes ps)
 
 -- | Parse an event stream from a lazy 'L.ByteString'.
 parseLBS :: MonadThrow m
          => ParseSettings
          -> L.ByteString
-         -> Producer m Event
-parseLBS ps lbs = CL.sourceList (L.toChunks lbs) =$= parseBytes ps
+         -> ConduitT i Event m ()
+parseLBS ps lbs = sourceLazy lbs .| parseBytes ps
 
 data XmlException = XmlException
     { xmlErrorMessage :: String
@@ -967,24 +1020,24 @@ ignoreAttrs = AttrParser $ const $ Right ([], ())
 
 -- | Keep parsing elements as long as the parser returns 'Just'.
 many :: Monad m
-     => ConduitM Event o m (Maybe a)
-     -> ConduitM Event o m [a]
+     => ConduitT Event o m (Maybe a)
+     -> ConduitT Event o m [a]
 many i = manyIgnore i $ return Nothing
 
 -- | Like 'many' but discards the results without building an intermediate list.
 --
 -- Since 1.5.0
 many_ :: MonadThrow m
-      => ConduitM Event o m (Maybe a)
-      -> ConduitM Event o m ()
+      => ConduitT Event o m (Maybe a)
+      -> ConduitT Event o m ()
 many_ consumer = manyIgnoreYield (return Nothing) (void <$> consumer)
 
 -- | Keep parsing elements as long as the parser returns 'Just'
 --   or the ignore parser returns 'Just'.
 manyIgnore :: Monad m
-           => ConduitM Event o m (Maybe a)
-           -> ConduitM Event o m (Maybe b)
-           -> ConduitM Event o m [a]
+           => ConduitT Event o m (Maybe a)
+           -> ConduitT Event o m (Maybe b)
+           -> ConduitT Event o m [a]
 manyIgnore i ignored = go id where
   go front = i >>= maybe (onFail front) (\y -> go $ front . (:) y)
   -- onFail is called if the main parser fails
@@ -993,25 +1046,25 @@ manyIgnore i ignored = go id where
 -- | Like @many@, but any tags and content the consumer doesn't match on
 --   are silently ignored.
 many' :: MonadThrow m
-      => ConduitM Event o m (Maybe a)
-      -> ConduitM Event o m [a]
+      => ConduitT Event o m (Maybe a)
+      -> ConduitT Event o m [a]
 many' consumer = manyIgnore consumer ignoreAllTreesContent
 
 
 -- | Like 'many', but uses 'yield' so the result list can be streamed
 --   to downstream conduits without waiting for 'manyYield' to finish
 manyYield :: Monad m
-          => ConduitM a b m (Maybe b)
-          -> Conduit a m b
+          => ConduitT a b m (Maybe b)
+          -> ConduitT a b m ()
 manyYield consumer = fix $ \loop ->
   consumer >>= maybe (return ()) (\x -> yield x >> loop)
 
 -- | Like 'manyIgnore', but uses 'yield' so the result list can be streamed
 --   to downstream conduits without waiting for 'manyIgnoreYield' to finish
 manyIgnoreYield :: MonadThrow m
-                => ConduitM Event b m (Maybe b) -- ^ Consuming parser that generates the result stream
-                -> ConduitM Event b m (Maybe ()) -- ^ Ignore parser that consumes elements to be ignored
-                -> Conduit Event m b
+                => ConduitT Event b m (Maybe b) -- ^ Consuming parser that generates the result stream
+                -> ConduitT Event b m (Maybe ()) -- ^ Ignore parser that consumes elements to be ignored
+                -> ConduitT Event b m ()
 manyIgnoreYield consumer ignoreParser = fix $ \loop ->
   consumer >>= maybe (onFail loop) (\x -> yield x >> loop)
   where onFail loop = ignoreParser >>= maybe (return ()) (const loop)
@@ -1019,8 +1072,8 @@ manyIgnoreYield consumer ignoreParser = fix $ \loop ->
 -- | Like 'many'', but uses 'yield' so the result list can be streamed
 --   to downstream conduits without waiting for 'manyYield'' to finish
 manyYield' :: MonadThrow m
-           => ConduitM Event b m (Maybe b)
-           -> Conduit Event m b
+           => ConduitT Event b m (Maybe b)
+           -> ConduitT Event b m ()
 manyYield' consumer = manyIgnoreYield consumer ignoreAllTreesContent
 
 
@@ -1029,7 +1082,7 @@ manyYield' consumer = manyIgnoreYield consumer ignoreAllTreesContent
 -- Returns @Just ()@ if a content 'Event' was consumed, @Nothing@ otherwise.
 --
 -- Since 1.5.0
-takeContent :: MonadThrow m => ConduitM Event Event m (Maybe ())
+takeContent :: MonadThrow m => ConduitT Event Event m (Maybe ())
 takeContent = do
   event <- await
   case event of
@@ -1049,7 +1102,7 @@ takeContent = do
 -- Returns @Just ()@ if an element was consumed, 'Nothing' otherwise.
 --
 -- Since 1.5.0
-takeTree :: MonadThrow m => NameMatcher a -> AttrParser b -> ConduitM Event Event m (Maybe ())
+takeTree :: MonadThrow m => NameMatcher a -> AttrParser b -> ConduitT Event Event m (Maybe ())
 takeTree nameMatcher attrParser = do
   event <- await
   case event of
@@ -1061,7 +1114,7 @@ takeTree nameMatcher attrParser = do
           endEvent <- await
           case endEvent of
             Just e'@(EventEndElement name') | name == name' -> yield e' >> return (Just ())
-            _ -> lift $ monadThrow $ InvalidEndElement name endEvent
+            _ -> lift $ throwM $ InvalidEndElement name endEvent
         _ -> leftover e >> return Nothing
       _ -> leftover e >> return Nothing
 
@@ -1076,63 +1129,45 @@ takeTree nameMatcher attrParser = do
 takeTreeContent :: MonadThrow m
                 => NameMatcher a
                 -> AttrParser b
-                -> ConduitM Event Event m (Maybe ())
+                -> ConduitT Event Event m (Maybe ())
 takeTreeContent nameMatcher attrParser = runMaybeT $ MaybeT (takeTree nameMatcher attrParser) <|> MaybeT takeContent
 
 -- | Like 'takeTreeContent', without checking for tag name or attributes.
 --
--- >>> runResourceT $ parseLBS def "text<a></a>" $$ takeAnyTreeContent =$= consume
+-- >>> runResourceT $ parseLBS def "text<a></a>" $$ takeAnyTreeContent .| consume
 -- Just [ EventContent (ContentText "text") ]
 --
--- >>> runResourceT $ parseLBS def "</a><b></b>" $$ takeAnyTreeContent =$= consume
+-- >>> runResourceT $ parseLBS def "</a><b></b>" $$ takeAnyTreeContent .| consume
 -- Just [ ]
 --
--- >>> runResourceT $ parseLBS def "<b><c></c></b></a>text" $$ takeAnyTreeContent =$= consume
+-- >>> runResourceT $ parseLBS def "<b><c></c></b></a>text" $$ takeAnyTreeContent .| consume
 -- Just [ EventBeginElement "b" [], EventBeginElement "c" [], EventEndElement "c", EventEndElement "b" ]
 --
 -- Since 1.5.0
 takeAnyTreeContent :: MonadThrow m
-                => ConduitM Event Event m (Maybe ())
+                => ConduitT Event Event m (Maybe ())
 takeAnyTreeContent = takeTreeContent anyName ignoreAttrs
 
 {-# DEPRECATED takeAllTreesContent "Please use 'takeAnyTreeContent'." #-}
-takeAllTreesContent :: MonadThrow m => ConduitM Event Event m (Maybe ())
+takeAllTreesContent :: MonadThrow m => ConduitT Event Event m (Maybe ())
 takeAllTreesContent = takeAnyTreeContent
 
-type DecodeEntities = Text -> Content
 
--- | Default implementation of 'DecodeEntities': handles numeric entities and
--- the five standard character entities (lt, gt, amp, quot, apos).
+-- | Default implementation of 'DecodeEntities', which leaves the
+-- entity as-is. Numeric character references and the five standard
+-- entities (lt, gt, amp, quot, pos) are handled internally by the
+-- parser.
 decodeXmlEntities :: DecodeEntities
-decodeXmlEntities "lt" = ContentText "<"
-decodeXmlEntities "gt" = ContentText ">"
-decodeXmlEntities "amp" = ContentText "&"
-decodeXmlEntities "quot" = ContentText "\""
-decodeXmlEntities "apos" = ContentText "'"
-decodeXmlEntities t = let backup = ContentEntity t in
-  case T.uncons t of
-    Just ('#', t') ->
-      case T.uncons t' of
-        Just ('x', t'')
-          | T.length t'' > 6 -> backup
-          | otherwise        -> decodeChar hexadecimal backup t''
-        _
-          | T.length t'  > 7 -> backup
-          | otherwise        -> decodeChar decimal backup t'
-    _ -> backup
+decodeXmlEntities = ContentEntity
 
--- | HTML4-compliant entity decoder. Handles numerics, the five standard
--- character entities, and the additional 248 entities defined by HTML 4 and
--- XHTML 1.
+-- | HTML4-compliant entity decoder. Handles the additional 248
+-- entities defined by HTML 4 and XHTML 1.
 --
 -- Note that HTML 5 introduces a drastically larger number of entities, and
 -- this code does not recognize most of them.
 decodeHtmlEntities :: DecodeEntities
 decodeHtmlEntities t =
-    case decodeXmlEntities t of
-        x@ContentText{} -> x
-        backup@ContentEntity{} ->
-            maybe backup ContentText $ Map.lookup t htmlEntities
+  maybe (ContentEntity t) ContentText $ Map.lookup t htmlEntities
 
 htmlEntities :: Map.Map T.Text T.Text
 htmlEntities = Map.fromList
@@ -1386,12 +1421,3 @@ htmlEntities = Map.fromList
     , ("hearts", "\9829")
     , ("diams", "\9830")
     ]
-
-decodeChar :: Reader Word32 -> Content -> Text -> Content
-decodeChar readNum backup = either (const backup) toContent . readNum
-  where
-    toContent (num, extra) | T.null extra =
-      case decodeUtf32BEWith ignore . toByteString $ fromWord32be num of
-          c    | T.length c    == 1 -> ContentText c
-               | otherwise          -> backup
-    toContent _ = backup
